/* 버튼 위젯 소스

   This file is part of otak-godori.

   Copyright (C) 2007 Otak-godori Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <gtk/gtk.h>

#include "../otak-lib/otak-util/otgtk.h"

#include "ogbutton.h"


static GtkDrawingAreaClass *og_button_parent_class = NULL;



/*
 * signal
 */
gboolean
og_button_enter_notify_event_cb (GtkWidget      *widget,
                                GdkEventButton  *event,
                                gpointer         user_data)
{
    og_button_change_type (OG_BUTTON (widget), OG_BUTTON_TYPE_OVER);
    return FALSE;
}

gboolean
og_button_leave_notify_event_cb (GtkWidget      *widget,
                                GdkEventButton  *event,
                                gpointer         user_data)
{
    og_button_change_type (OG_BUTTON (widget), OG_BUTTON_TYPE_NORMAL);
    return FALSE;
}

static gboolean
on_buton_expose_event_cb (GtkWidget         *widget,
                          GdkEventExpose    *event,
                          gpointer           user_data)
{
    OgButton *button = OG_BUTTON (widget);

    gdk_draw_pixbuf (widget->window,
            widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
            button->pixbufs[button->curr_type], 0, 0, 0, 0,
            button->width, button->height,
            GDK_RGB_DITHER_NORMAL, 0, 0);

    return FALSE;
}



/*
 * private
 */

static void
og_button_destroy (GtkObject *object)
{
    OgButton *button = OG_BUTTON (object);

    int i = 0;
    for (i = 0; i < OG_BUTTON_TYPE_COUNT; i++)
    {
        if (button->pixbufs[i] != NULL)
        {
            g_object_unref (button->pixbufs[i]);
            button->pixbufs[i] = NULL;
        }
    }

    (* GTK_OBJECT_CLASS (og_button_parent_class)->destroy) (object);
}

/* class init */
static void
og_button_class_init (OgButtonClass *klass)
{

    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);

    og_button_parent_class = g_type_class_peek_parent (klass);

    // 이곳에 연결하면 gtk_widget_destroy시 함께 실행된다.
    object_class->destroy = og_button_destroy;
}

/* instance init */
static void
og_button_init (OgButton *button)
{
    button->curr_type   = OG_BUTTON_TYPE_NORMAL;
    button->width       = -1;
    button->height      = -1;
    button->disable     = FALSE;

    int i = 0;
    for (i = 0; i < OG_BUTTON_TYPE_COUNT; i++)
    {
        button->pixbufs[i] = NULL;
    }

    gtk_widget_show (GTK_WIDGET (button));

    g_signal_connect ((gpointer) button, "expose_event",
            G_CALLBACK (on_buton_expose_event_cb), (gpointer) button);
    g_signal_connect ((gpointer) button, "enter_notify_event",
            G_CALLBACK (og_button_enter_notify_event_cb), (gpointer) button);
    g_signal_connect ((gpointer) button, "leave_notify_event",
            G_CALLBACK (og_button_leave_notify_event_cb), (gpointer) button);

    gtk_widget_set_events (GTK_WIDGET (button),
            GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
            GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
}


/* get type */
GType
og_button_get_type (void)
{

    static GType widget_type = 0;

    if (G_UNLIKELY (widget_type == 0))
    {
        const GTypeInfo widget_info =
        {
            sizeof (OgButtonClass),
            NULL,       /* base_init */
            NULL,
            (GClassInitFunc) og_button_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OgButton),
            0,          /* n_preallocs */
            (GInstanceInitFunc) og_button_init,
            NULL,       /* value_table */
        };
        widget_type = g_type_register_static (GTK_TYPE_DRAWING_AREA, "OgButton",
                &widget_info, 0);
    }

    return widget_type;
}



/*
 * public
 */

OgButton*
og_button_new_from_file (const gchar    *filename,
                         gint            width,
                         gint            height)
{
    g_assert (filename != NULL);
    g_assert (width > 0 && height > 0);

    OgButton *button = OG_BUTTON (g_object_new (OG_TYPE_BUTTON, NULL));

    if (button == NULL)
    {
        return NULL;
    }

    /* 프로퍼티 셋팅 */
    button->width = width;
    button->height = height;
    gtk_widget_set_size_request (GTK_WIDGET (button), width, height);

    /* 이미지 로딩 */
    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
    gint pixbuf_width = gdk_pixbuf_get_width (pixbuf);
    gint pixbuf_height = gdk_pixbuf_get_height (pixbuf);

    /* 필수 이미지 4개. 해당 스펙에 맞는 이미지가 아니면 오류. */
    g_assert (pixbuf_width == width * OG_BUTTON_TYPE_COUNT && pixbuf_height == height);

    /* 이미지 를 4등분 하여 넣는다 */
    int i = 0;
    for (i = 0; i < OG_BUTTON_TYPE_COUNT; i++)
    {
        button->pixbufs[i] = gdk_pixbuf_new_subpixbuf (pixbuf,
                i * width, 0, width, height);
    }

    g_object_unref (pixbuf);

    return button;
}

OgButton*
og_button_free (OgButton *button)
{
    return OG_BUTTON (ot_gtk_widget_free (GTK_WIDGET (button)));
}

void
og_button_change_type (OgButton     *button,
                       OgButtonType  type)
{
    OgButtonType old_type = button->curr_type;

    button->curr_type = button->disable ? OG_BUTTON_TYPE_DISABLE : type;
    if (button->curr_type != old_type) {
        gdk_window_invalidate_rect (GTK_WIDGET (button)->window, NULL, FALSE);
    }
}
